module Oneds
  module Mrproper
    module Cleaners
      class DirectoryCleaner < Oneds::Mrproper::Cleaners::Cleaner
        attr_reader :datastore_manager

        def initialize(client, dry_run)
          super(dry_run)

          @datastore_manager = Oneds::Mrproper::One::Managers::DatastoreManager.new client
        end

        def clean
          logger.info 'Cleaning forgotten datastore directories'
          datastores = datastore_manager.datastore_paths.sort
          base_path = File.dirname(datastores.first)
          logger.debug "Datasores registered in OpenNebula: #{datastores.inspect}"
          directories = Dir[File.join(base_path, '*')].sort
          logger.debug "Directories present on filesystem: #{directories.inspect}"

          dirs_to_remove = directories - datastores
          logger.debug "Directories that will be removed: #{dirs_to_remove.inspect}"

          remove_directories dirs_to_remove
        end

        private

        def remove_directories(directories)
          directories.each do |dir|
            unless File.directory? dir
              logger.warn "Skipping file #{dir.inspect}, not a directory"
              next
            end

            unless Dir[File.join(dir, '*')].empty?
              logger.warn "Skipping not empty directory #{dir.inspect} which is not registered in OpenNebula"
              next
            end

            logger.info "Removing empty directory #{dir.inspect}"
            FileUtils.rm_r dir unless dry_run
          end
        end
      end
    end
  end
end
