module Oneds
  module Mrproper
    module Cleaners
      class SystemCleaner < Oneds::Mrproper::Cleaners::Cleaner
        attr_reader :datastore_manager, :vm_manager

        def initialize(client, dry_run)
          super(dry_run)

          @datastore_manager = Oneds::Mrproper::One::Managers::DatastoreManager.new client
          @vm_manager = Oneds::Mrproper::One::Managers::VirtualMachineManager.new client
        end

        def clean
          logger.info 'Cleaning system datastores'
          datastores = datastore_manager.system_datastores
          datastores.each { |datastore| clean_datastore datastore }
        end

        def clean_datastore(datastore)
          ds_path = datastore['BASE_PATH']
          logger.info "Cleaning datastore #{datastore.name.inspect} with path #{ds_path.inspect}"

          unless File.exist? ds_path
            logger.warn "Path #{ds_path.inspect} not found on headnode, skipping"
            return
          end

          fs_vm_ids = Dir.entries(ds_path).select { |dir| dir =~ /^[0-9]*$/ }.sort
          logger.debug "Directories available on filesystem: #{fs_vm_ids.inspect}"

          remove_vm_directories fs_vm_ids, ds_path
        end

        private

        def remove_vm_directories(directories, ds_path)
          directories.each do |vm_id|
            vm = vm_manager.virtual_machine(vm_id)
            if vm && vm_manager.done?(vm)
              logger.info "Removing virtual machine directory with id #{vm.id.inspect}"
              FileUtils.rm_r File.join(ds_path, vm_id) unless dry_run
            end
          end
        end
      end
    end
  end
end
