require 'thor'
require 'yell'

module Oneds
  module Mrproper
    class CLI < Thor
      class_option :'dry-run',
                   default: false,
                   type: :boolean,
                   desc: 'Runs oneds-mrproper without modifying anything'
      class_option :debug,
                   default: false,
                   type: :boolean,
                   desc: 'Runs oneds-mrproper in debug mode'
      class_option :info,
                   default: false,
                   type: :boolean,
                   desc: 'Runs oneds-mrproper in info mode'

      method_option :system,
                    default: true,
                    type: :boolean,
                    desc: 'Clean also system datastores'
      method_option :image,
                    default: true,
                    type: :boolean,
                    desc: 'Clean also image datastores'
      method_option :directory,
                    default: true,
                    type: :boolean,
                    desc: 'Clean also forgotten datastore directories'
      method_option :'one-endpoint',
                    default: nil,
                    type: :string,
                    desc: 'OpenNebula endpoint'
      method_option :'one-secret',
                    default: nil,
                    type: :string,
                    desc: 'OpenNebula endpoint'

      desc 'clean', 'Cleaning datastores'
      def clean
        initialize_logger(options)
        client = Oneds::Mrproper::One::Helper.client options[:'one-secret'], options[:'one-endpoint']
        Oneds::Mrproper::Cleaners::SystemCleaner.new(client, options[:'dry-run']).clean if options[:system]
        Oneds::Mrproper::Cleaners::ImageCleaner.new(client, options[:'dry-run']).clean if options[:image]
        Oneds::Mrproper::Cleaners::DirectoryCleaner.new(client, options[:'dry-run']).clean if options[:directory]
      end

      default_task :clean

      private

      def initialize_logger(options)
        level = 'ERROR'
        level = 'INFO' if options[:info] || options[:'dry-run']
        level = 'DEBUG' if options[:debug]

        Yell.new :stdout, name: Object, level: level.downcase, format: Yell::DefaultFormat
        Object.send :include, Yell::Loggable

        logger.info 'DRY RUN ENABLED' if options[:'dry-run']
      end
    end
  end
end
